# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "calculate lake areas, count, and river length for different elevation bins"

import arcpy
import os
from arcpy import env
from arcpy.sa import *

def findField(fc, fi):
  lst = arcpy.ListFields(fc)
  for f in lst:
    if f.name == fi:
      return "found"
  return "not found"

arcpy.CheckOutExtension("Spatial")
env.workspace=r'E:\1.GrIS_river_evolution\water_mask\2016\Watson_Isortoq\0_river_and_lake_shapefiles'
#env.workspace=r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\0_river_and_lake_shapefiles\large_lake_and_river_shapefiles_for_paper'
env.overwriteOutput=True

eleBinFc = r'E:\1.GrIS_river_evolution\1_DEM\basin_watson_isortoq_30m_gimp_v2_reclassfication_polygon.shp'
outputpath = r'E:\1.GrIS_river_evolution\water_mask\2016\Watson_Isortoq\1_statistics'
if os.path.exists(outputpath)==False:
        os.mkdir(outputpath)
eleBinFc_with_info  =  outputpath + "\\" + "river_large_lake_statistics.shp"
tempFc1  = outputpath + "\\temp1.shp"
tempFc2  = outputpath + "\\temp2.shp"

sign = 0
featureclasses = arcpy.ListFeatureClasses()
for fc in featureclasses:
        print "process " + fc
        day = (fc.split("2015")[1])[0:3]
        if "lake" in fc:
                fieldmappings = arcpy.FieldMappings()
                if sign ==0:
                      fieldmappings.addTable(eleBinFc)
                elif sign%2==1:
                      fieldmappings.addTable(tempFc1)
                elif sign %2 == 0:
                      fieldmappings.addTable(tempFc2)
                countFieldMap=arcpy.FieldMap()
                countFieldMap.addInputField(fc,"count")
                areaFieldMap=arcpy.FieldMap()
                areaFieldMap.addInputField(fc,"area")
                countField=countFieldMap.outputField
                areaField=areaFieldMap.outputField
                countField.name=day + "Lcount"
                areaField.name=day + "Larea"
                countField.aliasName=day + "Lcount"
                areaField.aliasName=day + "Larea"
                countFieldMap.outputField=countField
                areaFieldMap.outputField=areaField
                countFieldMap.mergeRule="Sum"
                areaFieldMap.mergeRule="Sum"
                fieldmappings.addFieldMap(countFieldMap)
                fieldmappings.addFieldMap(areaFieldMap)
                if sign ==0:
                        arcpy.SpatialJoin_analysis(eleBinFc,fc,tempFc1,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                elif sign %2 == 1:
                        arcpy.SpatialJoin_analysis(tempFc1,fc,tempFc2,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                elif sign %2 == 0:
                        arcpy.SpatialJoin_analysis(tempFc2,fc,tempFc1,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                sign = sign + 1
        if "river" in fc:
                fieldmappings = arcpy.FieldMappings()
                if sign ==0:
                      fieldmappings.addTable(eleBinFc)
                elif sign%2==1:
                      fieldmappings.addTable(tempFc1)
                elif sign %2 == 0:
                      fieldmappings.addTable(tempFc2)
                fieldmappings.addTable(eleBinFc)
                lengthFieldMap=arcpy.FieldMap()
                lengthFieldMap.addInputField(fc,"length")
                lengthField=lengthFieldMap.outputField
                lengthField.name=day + "Rlength"
                lengthFieldMap.outputField=lengthField
                lengthFieldMap.mergeRule="Sum"
                fieldmappings.addFieldMap(lengthFieldMap)
                if sign ==0:
                        arcpy.SpatialJoin_analysis(eleBinFc,fc,tempFc1,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                elif sign %2 == 1:
                        arcpy.SpatialJoin_analysis(tempFc1,fc,tempFc2,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                elif sign %2 == 0:
                        arcpy.SpatialJoin_analysis(tempFc2,fc,tempFc1,"JOIN_ONE_TO_ONE","KEEP_ALL",fieldmappings,"INTERSECT")
                sign = sign + 1
if sign%2==0:
        arcpy.CopyFeatures_management(tempFc1, eleBinFc_with_info)
else:
        arcpy.CopyFeatures_management(tempFc2, eleBinFc_with_info)

arcpy.Delete_management(tempFc1)
arcpy.Delete_management(tempFc2)

discard=[]
fields=arcpy.ListFields(eleBinFc_with_info)
for field in fields:
        if "TARGET" in field.name or "Join" in field.name:
            discard.append(field.name)
        if "area" in field.name:
            name = field.name
            expression = "round(!" + name + "!/1000000,1)"
            arcpy.CalculateField_management(eleBinFc_with_info, name, expression, "PYTHON")
        if "length" in field.name:
            name = field.name
            expression = "round(!" + name + "!/1000,1)"
            arcpy.CalculateField_management(eleBinFc_with_info, name, expression, "PYTHON")
arcpy.CalculateField_management(eleBinFc_with_info, "area", "round(!SHAPE.area@SQUAREMETERS!/1000000,1)", "PYTHON")
arcpy.DeleteField_management(eleBinFc_with_info, discard)




                
                
                
        
        
        


        
        
        



                
            
