# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "calculate water volume from water mask and Pope's equation"

import arcpy
import os
from arcpy import env
import numpy as np
from arcpy.sa import *
from os import walk

#before run this code, run Pope's Jakobshavn_landsat_processing.m (D:\1. Codes and Programs\Landsat8_SupraglacialLakes-v1.0\allenpope-Landsat8_SupraglacialLakes-abe251a\Depth\Jakobshavn_landsat_processing.m) to create TOA reflectance images

watermask_file_path = r'E:\1.GrIS_river_evolution\water_mask\2016\Watson_Isortoq'
L8_image_path = r'E:\bda\westGrIS_2016\for_Isortoq_Watson'
outputPath = watermask_file_path + '\\water_depth'
outputTxtFile = outputPath + "\\water_volume.csv"

resolution = 30
bandIdentifier = "B13"  #B13, B24, different ways to derive lake depth base on different parameters, see Pope et al., 2014

if os.path.exists(outputPath)==False:
    os.mkdir(outputPath)
arcpy.CheckOutExtension("Spatial")
env.overwriteOutput=True
env.workspace=watermask_file_path
doyArray = np.array([])
areaArray =  np.array([])
volumeArray =  np.array([])

tiffs = arcpy.ListRasters("*","tif")
for tiff in tiffs:

    #if not '2016208' in tiff:
    #    continue

    name = tiff[:-4]
    L8_name = tiff.split('_')[1]
    doy = int(L8_name[-8:-5])
    print "process " + L8_name
    lakeMask = tiff
    refBand1 = L8_image_path + "\\" + L8_name + "\\" + L8_name + "_B1_ref.tif"
    refBand2 = L8_image_path + "\\" + L8_name + "\\" + L8_name + "_B2_ref.tif"
    refBand3 = L8_image_path + "\\" + L8_name + "\\" + L8_name + "_B3_ref.tif"
    refBand4 = L8_image_path + "\\" + L8_name + "\\" + L8_name + "_B4_ref.tif"

    if bandIdentifier == "B13":
        #coefficient for Band 1 and 3
        a = 0.1488
        b = 5.0370
        c = 5.0473
        bandA=refBand1
        bandB=refBand3
    elif bandIdentifier == "B24":
        #coefficient for Band 2 and 4
        a = 3.4414
        b = -9.0500
        c = 7.8243
        bandA=refBand2
        bandB=refBand4
    else:
        print "incorrect band input"
        break

    #crete depth raster
    #this depth raster can be created when this script is run first time
    depthName = outputPath + "\\" + name + "_depth_" + bandIdentifier + ".tif"
    if not os.path.exists(depthName):
        depthRaster = a + b * Ln(Float(bandA)/Float(bandB)) + c * Power(Ln(Float(bandA)/Float(bandB)),2)
        depthRaster.save(depthName)
        arcpy.CalculateStatistics_management(depthName)
        arcpy.BuildPyramids_management(depthName)
        print "depth raster created"
    
    #create depth for lakes
    depthLakeName = outputPath + "\\" + name + "_depth_" + bandIdentifier + "_for_waterMask.tif"
    if not os.path.exists(depthLakeName):
        depthLakeRaster = SetNull(lakeMask,Raster(depthName),"VALUE < 1")  # <1 OR >0, depending on the lake pixel values
        depthLakeRaster.save(depthLakeName)
        arcpy.CalculateStatistics_management(depthLakeName)
        arcpy.BuildPyramids_management(depthLakeName)
        print "lake mask created for depth"
        
    #calculate lake volume
    array = arcpy.RasterToNumPyArray(depthLakeName, nodata_to_value = 0)
    lakearea = float(np.count_nonzero(array)) * resolution * resolution /1000000 #lake area in km2
    lakeVolume = float(array.sum()) * resolution * resolution / 1000000000 #lake volume in km3
    print "lake area is " + str(lakearea)
    print "lake volume is " + str(lakeVolume)
    del array  #this is important, otherwise, Memory error will be returned

    doyArray = np.append(doyArray,[doy])
    areaArray =  np.append(areaArray,[lakearea])
    volumeArray =  np.append(volumeArray,[lakeVolume])

#export area, volume table
finalData = np.zeros(doyArray.size, dtype=[('var1', 'S6'), ('var2', float),('var3', float)]) #source: http://stackoverflow.com/questions/16621351/how-to-use-python-numpy-savetxt-to-write-strings-and-float-number-to-an-ascii-fi
finalData['var1'] = doyArray
finalData['var2'] = areaArray
finalData['var3'] = volumeArray
finalData = finalData[finalData['var1'].argsort()] #sort results based on DOY, source: http://stackoverflow.com/questions/2828059/sorting-arrays-in-numpy-by-column
headerInfo='doy,area,volume'
np.savetxt(outputTxtFile, finalData, fmt="%3s,%2.5f,%2.5f",delimiter=',',header=headerInfo)
