# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot lake/river information and MAR smb"
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from pandas.io.excel import ExcelWriter

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2

def create_waterarea_fraction_DF(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    resolution = 30
    waterfraction = waterarea*resolution*resolution/(1000000*area.values)
    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)
    return waterfraction

def create_watervolume_DF(input_zonalSta_watervolume_point_file):
    dbf = ps.open(input_zonalSta_watervolume_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterdepthIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterdepth = df.loc[waterdepthIdx]

    resolution = 30
    overestimation_rate = 0.8  #according to Pope, the spectral method overestimates water, use this coefficient to rectify the volume
    waterVolume = waterdepth * resolution * resolution / (1000000*area.values) * 1000 * overestimation_rate# water volume in mm
    waterVolume.index = [int(day) for day in waterdepthIdx]
    waterVolume = waterVolume.sort_index(ascending=True)
    return waterVolume

def create_SMB_DF(input_SMBExcelFile):
    filename, ext = os.path.splitext(input_SMBExcelFile)
    if ext == '.xlsx':
        data = pd.read_excel(input_SMBExcelFile)
    if ext == '.csv':
        data = pd.read_csv(input_SMBExcelFile)
    df = pd.DataFrame(data)
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    if ext == '.xlsx':
        days = df.index[:]
    if ext == '.csv':
        days = df.index[1:-1]  #In csv files, the first index is 'basin', which needs to be deleted
        df = df.iloc[1:-1]
        
    df.index = [int(day[3:]) for day in days]
    print df
    return df

def create_river_lake_DF(river_lake_stat_file):

    dbf = ps.open(river_lake_stat_file)
    d = dbf.by_col_array(dbf.header)  # my solution is better than the last sentence
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    types = df.index[:]
    areaIdx = 'area'
    lakeareaIdx = [ t for t in types if 'Larea' in t]
    lakecountIdx = [ t for t in types if 'Lcount' in t]
    riverIdx = [ t for t in types if 'Rlength' in t]

    #Use .iloc for integer based indexing and .loc for label based indexing.
    area = df.loc[areaIdx]
    lakearea = df.loc[lakeareaIdx]
    lakecount = df.loc[lakecountIdx]
    riverlength = df.loc[riverIdx]

    lakearea_dayIdx = lakearea.index[:]
    lakearea_dayIdx = [int(day[:3]) for day in lakearea_dayIdx]
    river_dayIdx = riverlength.index[:]
    river_dayIdx = [int(day[:3]) for day in river_dayIdx]

    lakefraction = lakearea/area.values
    drainage_density = riverlength/area.values

    lakecount.index = lakearea_dayIdx
    lakefraction.index = lakearea_dayIdx
    drainage_density.index = river_dayIdx

    lakecount = lakecount.sort_index(ascending=True)
    lakefraction = lakefraction.sort_index(ascending=True)
    drainage_density = drainage_density.sort_index(ascending=True)


    lakearea.index = lakearea_dayIdx
    lakearea = lakearea.sort_index(ascending=True)
    riverlength.index = river_dayIdx
    riverlength = riverlength.sort_index(ascending=True)

    return lakecount,lakearea,lakefraction,riverlength,drainage_density

def select_doy_accRunoff_from_all(df_runoff,acc_df_all_runoff):
    days = df_runoff.index[:]
    acc_doy_runoff = acc_df_all_runoff.loc[days]
    return acc_doy_runoff

def cal_lake_drainage_ratio(lakecount):
    lakeCount_DOY214 = lakecount.loc[214]
    print lakeCount_DOY214
    lakeCount_maximum = lakecount.max().to_frame().T
    print lakeCount_maximum

    #lakeCount_DOY214.loc['total'] = lakeCount_DOY214.sum()
    #lakeCount_maximum.loc['total'] = lakeCount_maximum.sum()


    drainage_ration = ((lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
    return drainage_ration


def plot_1(df_melt,waterfraction,lakefraction,riverfraction,lakecount,drainage_density):

    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)

    df_melt.plot(ax=axarr[0][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('SMB melt (mm/day)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))


    waterfraction.plot(ax=axarr[0][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][1].set_xlabel('DOY')
    axarr[0][1].set_ylabel('water area fraction')
    axarr[0][1].annotate('(b)',size = 20,xy=get_axis_limits(axarr[0][1]))
    axarr[0][1].set_ylim([0, 0.08])
    axarr[0][1].set_yticks(np.arange(0,0.1,0.02))


    lakefraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('lake area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))


    riverfraction.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('river area fraction')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[1][1].set_ylim([0, 0.08])
    axarr[1][1].set_yticks(np.arange(0,0.1,0.02))


    lakecount.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('lake count')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    axarr[2][0].set_yticks(np.arange(0,500,100))


    drainage_density.plot(ax=axarr[2][1],marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('drainage density')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[2][1].set_yticks(np.arange(0,1,0.2))


    axarr[2][1].legend(loc='upper right',fontsize=15)

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_2(df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount):
    
    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


    df_runoff.plot(ax=axarr[0][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('delayed R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 60])


    df_nd_runoff.plot(ax=axarr[0][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][1].set_xlabel('DOY')
    axarr[0][1].set_ylabel('non-delayed R (mm)')
    axarr[0][1].annotate('(b)',size = 20,xy=get_axis_limits(axarr[0][1]))
    axarr[0][1].set_ylim([0, 60])

    axarr[0][1].legend(loc='upper right',fontsize=15)


    waterfraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('water area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))

    riverfraction.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('river area fraction')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    axarr[1][1].set_ylim([0, 0.08])
    axarr[1][1].set_yticks(np.arange(0,0.1,0.02))


    lakefraction.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('lake area fraction')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[2][0].set_ylim([0, 0.08])
    axarr[2][0].set_yticks(np.arange(0,0.1,0.02))

    lakecount.plot(ax=axarr[2][1],legend=False,marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('lake count')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[2][1].set_yticks(np.arange(0,500,100))

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_3(df_all_runoff, df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount):
    
    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


    df_all_runoff.plot(ax=axarr[0][0],legend=False,cmap=cmap)
    df_runoff.plot(ax=axarr[0][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('delayed R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 70])
    axarr[0][0].set_xlim([160, 240])


    df_all_nd_runoff.plot(ax=axarr[0][1],legend=False,cmap=cmap)
    df_nd_runoff.plot(ax=axarr[0][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][1].set_xlabel('DOY')
    axarr[0][1].set_ylabel('non-delayed R (mm)')
    axarr[0][1].annotate('(b)',size = 20,xy=get_axis_limits(axarr[0][1]))
    axarr[0][1].set_ylim([0, 70])
    axarr[0][1].set_xlim([160, 240])

    waterfraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('water area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))
    axarr[1][0].set_xlim([160, 240])

    riverfraction.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('river area fraction')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    axarr[1][1].set_ylim([0, 0.08])
    axarr[1][1].set_yticks(np.arange(0,0.1,0.02))
    axarr[1][1].set_xlim([160, 240])


    lakefraction.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('lake area fraction')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[2][0].set_ylim([0, 0.08])
    axarr[2][0].set_yticks(np.arange(0,0.1,0.02))
    axarr[2][0].set_xlim([160, 240])

    diff = riverfraction - lakefraction
    diff.plot(ax=axarr[2][1],legend=False,marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('lake count')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    #axarr[2][1].set_yticks(np.arange(0,500,100))
    axarr[2][1].set_xlim([160, 240])

    #axarr[2][1].legend(loc='upper right',fontsize=15)

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_4(df_all_runoff, df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount):

    f, ax1 = plt.subplots()
    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)

    df_all_runoff.plot(ax=ax1,legend=False,cmap=cmap)
    #df_runoff.plot(ax=ax1,legend=False,linestyle="None",marker = 'o',cmap=cmap)
    ax1.set_ylabel('delayed R (mm)')
    ax1.set_xlabel('DOY')
    ax1.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
    ax1.set_ylim([0, 70])
    ax1.set_xlim([160, 240])

    ax2 = ax1.twinx() 
    waterfraction.plot(ax=ax2,legend=False,linestyle="None",marker = 'o',cmap=cmap)
    ax2.set_ylabel('water area fraction')
    ax2.annotate('(b)',size = 20,xy=get_axis_limits(ax2))
    ax2.set_ylim([0, 0.08])
    ax2.set_yticks(np.arange(0,0.1,0.02))
    ax2.set_xlim([160, 240])

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_5(df_all_runoff, df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction,lakecount):
    
    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


    df_all_runoff = df_all_runoff.cumsum()

    df_all_runoff.plot(ax=axarr[0][0],legend=False,cmap=cmap)
    #df_runoff.plot(ax=axarr[0][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('delayed R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 800])
    axarr[0][0].set_xlim([160, 240])

    df_all_nd_runoff[df_all_nd_runoff<0]=0
    df_all_nd_runoff = df_all_nd_runoff.cumsum()

    df_all_nd_runoff.plot(ax=axarr[0][1],legend=False,cmap=cmap)
    #df_nd_runoff.plot(ax=axarr[0][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][1].set_xlabel('DOY')
    axarr[0][1].set_ylabel('non-delayed R (mm)')
    axarr[0][1].annotate('(b)',size = 20,xy=get_axis_limits(axarr[0][1]))
    axarr[0][1].set_ylim([0, 800])
    axarr[0][1].set_xlim([160, 240])

    waterfraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('water area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))
    axarr[1][0].set_xlim([160, 240])

    watervolume.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('water storage (mm)')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    #axarr[1][1].set_ylim([0, 0.08])
    #axarr[1][1].set_yticks(np.arange(0,0.1,0.02))
    #axarr[1][1].set_xlim([160, 240])


    lakefraction.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('lake area fraction')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[2][0].set_ylim([0, 0.08])
    axarr[2][0].set_yticks(np.arange(0,0.1,0.02))
    axarr[2][0].set_xlim([160, 240])

    diff = riverfraction - lakefraction
    diff.plot(ax=axarr[2][1],legend=False,marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('lake count')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    #axarr[2][1].set_yticks(np.arange(0,500,100))
    axarr[2][1].set_xlim([160, 240])

    #axarr[2][1].legend(loc='upper right',fontsize=15)

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_6(df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction):
    
    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


    df_runoff.plot(ax=axarr[0][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('delayed R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 60])

    df_nd_runoff.plot(ax=axarr[0][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[0][1].set_xlabel('DOY')
    axarr[0][1].set_ylabel('non-delayed R (mm)')
    axarr[0][1].annotate('(b)',size = 20,xy=get_axis_limits(axarr[0][1]))
    axarr[0][1].set_ylim([0, 60])

    axarr[0][1].legend(loc='upper right',fontsize=15)


    waterfraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('water area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))

    watervolume.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('water storage (mm)')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    #axarr[1][1].set_ylim([0, 0.08])
    axarr[1][1].set_yticks(np.arange(0,220,40))


    riverfraction.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('river area fraction')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[2][0].set_ylim([0, 0.08])
    axarr[2][0].set_yticks(np.arange(0,0.1,0.02))

    lakefraction.plot(ax=axarr[2][1],legend=False,marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('lake area fraction')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[2][1].set_ylim([0, 0.08])
    axarr[2][1].set_yticks(np.arange(0,0.1,0.02))

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_scatter_1(df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction):

    #set plotting environments
    matplotlib.style.use('classic')
    matplotlib.rcParams.update({'font.size': 30})
    matplotlib.rcParams['axes.linewidth'] = 2
    matplotlib.rcParams['xtick.major.size'] = 30
    matplotlib.rcParams['xtick.major.width'] = 2
    matplotlib.rcParams['ytick.major.size'] = 39
    matplotlib.rcParams['ytick.major.width'] = 2
    matplotlib.rcParams['lines.linewidth'] = 3
    matplotlib.rcParams['lines.markersize'] = 200
    f, ax1 = plt.subplots()
    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    #cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)

    color=iter(cm.bwr_r(np.linspace(0,1,8)))

    df_nd_runoff = df_nd_runoff.cumsum()

    i = 0
    c = next(color)
    for column in waterfraction:
        if i == 3:
            c = next(color)  #to match colors
        ax1.scatter(df_nd_runoff[column],watervolume[column],s=300,marker='o',c=c)
        c = next(color)
        i = i + 1

    ax1.set_ylabel('water storage (mm)')
    ax1.set_xlabel('surface runoff (mm)')
    ax1.legend(loc='upper right',fontsize=30)

    #ax2 = ax1.twinx() 
    #waterfraction.plot(ax=ax2,legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #ax2.set_ylabel('water area fraction')
    #ax2.annotate('(b)',size = 20,xy=get_axis_limits(ax2))
    #ax2.set_ylim([0, 0.08])
    #ax2.set_yticks(np.arange(0,0.1,0.02))
    #ax2.set_xlim([160, 240])

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_final(df_runoff,df_all_runoff,waterfraction,watervolume,riverfraction,lakefraction):
    f, axarr = plt.subplots(3,2)

    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


    df_all_runoff.plot(ax=axarr[0][0],legend=False,cmap=cmap)
    df_runoff.plot(ax=axarr[0][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 60])
    axarr[0][0].set_xlim([160, 240])


    df_all_runoff = df_all_runoff.cumsum()
    df_acc_runoff = select_doy_accRunoff_from_all(df_runoff,df_all_runoff)
    df_all_runoff.plot(ax=axarr[0][1],legend=False,cmap=cmap)
    df_acc_runoff.plot(ax=axarr[0][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][1].set_ylabel('accumulative R (mm)')
    axarr[0][1].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][1].set_ylim([0, 600])
    axarr[0][1].set_yticks(np.arange(0,800,200))
    axarr[0][1].set_xlim([160, 240])

    waterfraction.plot(ax=axarr[1][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('water area fraction')
    axarr[1][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 0.08])
    axarr[1][0].set_yticks(np.arange(0,0.1,0.02))

    watervolume.plot(ax=axarr[1][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[1][1].set_ylabel('water storage (mm)')
    axarr[1][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    axarr[1][1].set_ylim([0, 160])
    axarr[1][1].set_yticks(np.arange(0,180,40))


    riverfraction.plot(ax=axarr[2][0],legend=False,marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('river area fraction')
    axarr[2][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[2][0].set_ylim([0, 0.08])
    axarr[2][0].set_yticks(np.arange(0,0.1,0.02))

    lakefraction.plot(ax=axarr[2][1],legend=False,marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('lake area fraction')
    axarr[2][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[2][1].set_ylim([0, 0.08])
    axarr[2][1].set_yticks(np.arange(0,0.1,0.02))

    axarr[2][1].legend(loc='upper right',fontsize=15)

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

def plot_lake_drainage_averageArea(lakecount,drainage_ratio,ave_lake_area):

    #set plotting environments
    matplotlib.style.use('classic')
    matplotlib.rcParams.update({'font.size': 30})
    matplotlib.rcParams['axes.linewidth'] = 2
    matplotlib.rcParams['xtick.major.size'] = 20
    matplotlib.rcParams['xtick.major.width'] = 2
    matplotlib.rcParams['ytick.major.size'] = 20
    matplotlib.rcParams['ytick.major.width'] = 2
    matplotlib.rcParams['lines.linewidth'] = 3
    matplotlib.rcParams['lines.markersize'] = 20

    f, (ax1,ax2) = plt.subplots(2)

    lakecount.plot(ax=ax1,legend=False,marker = 'o',cmap=cmap)
    ax1.set_xlabel('DOY')
    ax1.set_ylabel('lake count')
    ax1.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
    ax1.set_yticks(np.arange(0,500,100))
    ax1.legend(loc='upper right',fontsize=25)


    x=[800,1000,1200,1400,1600,1800]

    ax2.plot(x,drainage_ratio.loc[0].values,marker = 'd', color = 'r',zorder=10)
    #ax2.set_ylim([0, 1.2])
    ax2.set_xlim([600, 2000])
    #ax2.set_yticks(np.arange(0,1.2,0.2))
    ax2.set_xticks(np.arange(600,2000,200))
    ax2.set_ylabel('lake disappearance ratio', color = 'r')
    ax2.set_xlabel('elevation (m)')
    ax2.yaxis.label.set_color('r')
    ax2.tick_params(axis='y', colors='r')


    ax21 = ax2.twinx() 
    (_, caps, _) = ax21.errorbar(x,ave_lake_area.loc['ave_area'].values,yerr= ave_lake_area.loc['std_area'].values, capsize=10, elinewidth=3, marker = 's', color = 'b')
    for cap in caps:
        cap.set_markeredgewidth(3)
    ax21.set_ylabel('average lake area $\mathregular{(km^2)}$', color = 'b')
    ax21.yaxis.label.set_color('b')
    ax21.tick_params(axis='y', colors='b')

    ax21.axhline(y=0.25,color='k', linestyle='--',linewidth=2,zorder=0)

    plt.show()
        
    f.savefig(r'E:\lake_drainage_2015.tif', format='tif', dpi=450)



#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 30})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 16
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 16
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 10


#output of step 7
zonalSta_waterarea_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_waterarea.dbf'

#output of step 11
river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_lake_statistics.dbf'

#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'


#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
meltFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'

#ouput of step 15
zonalSta_watervolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_watervolume.dbf'


lakecount,lakearea,lakefraction,riverlength,drainage_density = create_river_lake_DF(river_lake_stat_file)
waterfraction = create_waterarea_fraction_DF(zonalSta_waterarea_point_file)
riverfraction = waterfraction - lakefraction #a key step
riverfraction[riverfraction<0]=0


df_runoff = create_SMB_DF(runoffFile)
df_melt = create_SMB_DF(meltFile)
df_nd_runoff = create_SMB_DF(nd_runoffFile)
df_nd_runoff[df_nd_runoff<0]=0


df_all_runoff = create_SMB_DF(all_runoffFile)
df_all_melt = create_SMB_DF(all_meltFile)
df_all_nd_runoff = create_SMB_DF(all_nd_runoffFile)
df_all_nd_runoff[df_nd_runoff<0]=0


#calculate water volume (lake volume + river volume)
watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)


#calculate lake drainage ratio  = (lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
drainage_ratio = cal_lake_drainage_ratio(lakecount)

#calculate average lake area
ave_lake_area = (lakearea/lakecount)
ave_lake_area.loc['ave_area']=ave_lake_area.mean()
ave_lake_area.loc['std_area']=ave_lake_area.std()

print ave_lake_area.loc['ave_area']
print ave_lake_area.loc['std_area']


matplotlib.rcParams.update({'font.size': 24})
matplotlib.rcParams['xtick.major.size'] = 24
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 24
matplotlib.rcParams['ytick.major.width'] = 2
cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


#plot_scatter_1(df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction)

#plot_2(df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount)

#plot_3(df_all_runoff,df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount)

#plot_4(df_all_runoff,df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,riverfraction,lakefraction,lakecount)

#plot_5(df_all_runoff,df_all_nd_runoff,df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction,lakecount)

#plot_6(df_runoff,df_nd_runoff,waterfraction,watervolume,riverfraction,lakefraction)

#plot_final(df_runoff,df_all_runoff,waterfraction,watervolume,riverfraction,lakefraction)

plot_lake_drainage_averageArea(lakecount,drainage_ratio,ave_lake_area)












