# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "seprate lake and river volume in order to understand the contributions of lakes and rivers to total water volume"


import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from textwrap import wrap

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2


def create_watervolume_DF(input_zonalSta_watervolume_point_file):
    dbf = ps.open(input_zonalSta_watervolume_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterdepthIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterdepth = df.loc[waterdepthIdx]

    resolution = 30
    overestimation_rate = 0.8  #according to Pope, the spectral method overestimates water, use this coefficient to rectify the volume
    waterVolume = waterdepth * resolution * resolution / (1000000*area.values) * 1000 * overestimation_rate# water volume in mm
    waterVolume.index = [int(day) for day in waterdepthIdx]
    waterVolume = waterVolume.sort_index(ascending=True)
    return waterVolume


#ouput of step 15
zonalSta_watervolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_watervolume.dbf'

#ouput of step 20
zonalSta_lakevolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_lake_volume.dbf'


df_watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)

df_lakevolume = create_watervolume_DF(zonalSta_lakevolume_point_file)


df_rivervolume = df_watervolume - df_lakevolume

print df_watervolume

print df_lakevolume


print df_lakevolume/df_watervolume



#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 18})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 16
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 16
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 10


#f, axarr = plt.subplots(3,2)

f, (ax1, ax2, ax3) = plt.subplots(3, sharex=True, sharey=True)


#f, axarr = plt.subplots(2,2)
#Google: How to use colormaps to color plots of Pandas DataFrames
#also see colormaps_reference
#also see Invert colormap in matplotlib
cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)


df_watervolume.plot(ax=ax1,legend=False,marker = 'o',cmap=cmap)
#ax1.set_xlabel('DOY')
ax1.set_ylabel('water volume (mm)')
ax1.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
ax1.set_ylim([0, 140])
#ax1.set_xlim([160, 240])

df_lakevolume.plot(ax=ax2,legend=False,marker = 'o',cmap=cmap)
#ax2.set_xlabel('DOY')
ax2.set_ylabel('lake volume (mm)')
ax2.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
ax2.set_ylim([0, 140])
#ax1.set_xlim([160, 240])

df_rivervolume.plot(ax=ax3,legend=False,marker = 'o',cmap=cmap)
ax3.set_xlabel('DOY')
ax3.set_ylabel('river volume (mm)')
ax3.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
ax3.set_ylim([0, 140])
#ax1.set_xlim([160, 240])

ax3.legend(loc='upper right',fontsize=15)

f.subplots_adjust(hspace=0)
plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)

plt.show()

f.savefig(r'E:\water_lake_river_volume_2015.tif', format='tif', dpi=450)
