# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot lake/river information and MAR smb"
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from pandas.io.excel import ExcelWriter

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2

def create_waterarea_fraction_DF(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    resolution = 30
    waterfraction = waterarea*resolution*resolution/(1000000*area.values)
    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)

    waterarea = waterarea*resolution*resolution/(1000000)
    waterarea.index = [int(day) for day in waterareaIdx]
    waterarea = waterarea.sort_index(ascending=True)


    return waterarea, waterfraction

def create_watervolume_DF(input_zonalSta_watervolume_point_file):
    dbf = ps.open(input_zonalSta_watervolume_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterdepthIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterdepth = df.loc[waterdepthIdx]

    resolution = 30
    overestimation_rate = 0.8  #according to Pope, the spectral method overestimates water, use this coefficient to rectify the volume
    #waterVolume = waterdepth * resolution * resolution / (1000000*area.values) * 1000 * overestimation_rate# water volume in mm

    waterVolume = waterdepth * resolution * resolution / 1000000000 * overestimation_rate# water volume in km3, for paper table only

    waterVolume.index = [int(day) for day in waterdepthIdx]
    waterVolume = waterVolume.sort_index(ascending=True)
    return waterVolume

def create_SMB_DF(input_SMBExcelFile):
    filename, ext = os.path.splitext(input_SMBExcelFile)
    if ext == '.xlsx':
        data = pd.read_excel(input_SMBExcelFile)
    if ext == '.csv':
        data = pd.read_csv(input_SMBExcelFile)
    df = pd.DataFrame(data)
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    if ext == '.xlsx':
        days = df.index[:]
    if ext == '.csv':
        days = df.index[1:-1]  #In csv files, the first index is 'basin', which needs to be deleted
        df = df.iloc[1:-1]
        
    df.index = [int(day[3:]) for day in days]
    print df
    return df

def create_river_lake_DF(river_lake_stat_file):

    dbf = ps.open(river_lake_stat_file)
    d = dbf.by_col_array(dbf.header)  # my solution is better than the last sentence
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    types = df.index[:]
    areaIdx = 'area'
    lakeareaIdx = [ t for t in types if 'Larea' in t]
    lakecountIdx = [ t for t in types if 'Lcount' in t]
    riverIdx = [ t for t in types if 'Rlength' in t]

    #Use .iloc for integer based indexing and .loc for label based indexing.
    area = df.loc[areaIdx]
    lakearea = df.loc[lakeareaIdx]
    lakecount = df.loc[lakecountIdx]
    riverlength = df.loc[riverIdx]

    lakearea_dayIdx = lakearea.index[:]
    lakearea_dayIdx = [int(day[:3]) for day in lakearea_dayIdx]
    river_dayIdx = riverlength.index[:]
    river_dayIdx = [int(day[:3]) for day in river_dayIdx]

    lakefraction = lakearea/area.values
    drainage_density = riverlength/area.values

    lakecount.index = lakearea_dayIdx
    lakefraction.index = lakearea_dayIdx
    drainage_density.index = river_dayIdx

    lakecount = lakecount.sort_index(ascending=True)
    lakefraction = lakefraction.sort_index(ascending=True)
    drainage_density = drainage_density.sort_index(ascending=True)


    lakearea.index = lakearea_dayIdx
    lakearea = lakearea.sort_index(ascending=True)
    riverlength.index = river_dayIdx
    riverlength = riverlength.sort_index(ascending=True)

    return lakecount,lakearea,lakefraction,riverlength,drainage_density

def select_doy_Runoff_from_all(df_runoff,df_all_runoff):
    doys = df_runoff.index[:]
    df_runoff = df_all_runoff.loc[doys]
    return df_runoff

def lake_drainage_ratio(lakecount):
    lakeCount_DOY214 = lakecount.loc[214]
    print lakeCount_DOY214
    lakeCount_maximum = lakecount.max().to_frame().T
    print lakeCount_maximum

    #lakeCount_DOY214.loc['total'] = lakeCount_DOY214.sum()
    #lakeCount_maximum.loc['total'] = lakeCount_maximum.sum()


    drainage_ration = ((lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
    return drainage_ration

def create_SMB_DF(input_SMBExcelFile):
    filename, ext = os.path.splitext(input_SMBExcelFile)
    if ext == '.xlsx':
        data = pd.read_excel(input_SMBExcelFile)
    if ext == '.csv':
        data = pd.read_csv(input_SMBExcelFile)
    df = pd.DataFrame(data)
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    if ext == '.xlsx':
        days = df.index[:]
    if ext == '.csv':
        days = df.index[1:-1]  #In csv files, the first index is 'basin', which needs to be deleted
        df = df.iloc[1:-1]
        
    df.index = [int(day[3:]) for day in days]
    print df
    return df

def create_all_SMB_DF(input_smb_file,area):
    data = pd.read_excel(input_smb_file)
    df = pd.DataFrame(data)
    df['DOY'] = df.index
    cols = list(df)
    mar_cols = [c for c in cols if 'mar' in c]
    racmo_cols = [c for c in cols if 'racmo' in c]
    merra_cols = [c for c in cols if 'merra' in c]

    mar_cols.insert(0,'DOY')
    racmo_cols.insert(0,'DOY')
    merra_cols.insert(0,'DOY')

    df_mar = df[mar_cols]
    df_racmo = df[racmo_cols]
    df_merra = df[merra_cols]
    df_mar.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_racmo.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_merra.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_mar.set_index('DOY', inplace=True)
    df_racmo.set_index('DOY', inplace=True)
    df_merra.set_index('DOY', inplace=True)

    df_mar = df_mar /(area * 1000000)*3600*1000  #chagne m3/s/day into mm/day, note, 24 hour day has been added previously
    df_racmo = df_racmo /(area * 1000000)*3600*1000
    df_merra = df_merra /(area * 1000000)*3600*1000

    return df_mar,df_racmo,df_merra

def get_eleBin_area(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    area = df.loc[areaIdx]

    return area

#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 30})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 16
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 16
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 10


#output of step 7
zonalSta_waterarea_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_waterarea.dbf'

#output of step 11
river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_lake_statistics.dbf'

#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'

#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
meltFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'

#ouput of step 15
zonalSta_watervolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_watervolume.dbf'

#ouput of step 20
zonalSta_lakevolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_lake_volume.dbf'

# run E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse, step 1-4 to get runoff for all SMB models
# note run SMB_Step4_calculate_smb_runoff_for_specific_days_new.py!! 2018.03.29
input_smb_file = r'E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse\output_smb_runoff_output_for_DOYs.xlsx'

df_watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)
df_lakevolume = create_watervolume_DF(zonalSta_lakevolume_point_file)
df_rivervolume = df_watervolume - df_lakevolume

lakecount,lakearea,lakefraction,riverlength,drainage_density = create_river_lake_DF(river_lake_stat_file)
waterarea,waterfraction = create_waterarea_fraction_DF(zonalSta_waterarea_point_file)
riverfraction = waterfraction - lakefraction #a key step
riverfraction[riverfraction<0]=0
riverarea = waterarea - lakearea
riverarea[riverarea<0]=0

df_runoff = create_SMB_DF(runoffFile)
df_melt = create_SMB_DF(meltFile)
df_nd_runoff = create_SMB_DF(nd_runoffFile)
df_nd_runoff[df_nd_runoff<0]=0


df_all_runoff = create_SMB_DF(all_runoffFile)
df_all_melt = create_SMB_DF(all_meltFile)
df_all_nd_runoff = create_SMB_DF(all_nd_runoffFile)
df_all_nd_runoff[df_nd_runoff<0]=0

area = get_eleBin_area(zonalSta_waterarea_point_file)
df_mar,df_racmo,df_merra = create_all_SMB_DF(input_smb_file,area)




#----------------------------for paper calculation-----------------------------#


#for smb-modeled runoff in table 1
df_merra = df_merra.cumsum()
df_merra_doy = select_doy_Runoff_from_all(df_runoff,df_merra)
print df_merra_doy
print 'done'


#for water area calculation in paper, table 1. 
print waterarea
print waterarea.sum(axis = 1)


print lakearea
print riverarea


#calculate water volume (lake volume + river volume)
watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)


#for water storage calculation in paper, table 1. Notice to change the unit of water volume in create_watervolume_DF into km3
print watervolume
print watervolume.sum(axis = 1)
print 'done'



#calculate lake drainage ratio ( = maximum lake count / lake count on DOY 214)
drainage_ratio = lake_drainage_ratio(lakecount)

#calculate average lake area
ave_lake_area = (lakearea/lakecount)
ave_lake_area.loc['ave_area']=ave_lake_area.mean()
ave_lake_area.loc['std_area']=ave_lake_area.std()

#print ave_lake_area.loc['ave_area']
#print ave_lake_area.loc['std_area']



###-------------------calculate river fraction contribution to water fraction
#print lakefraction
#print waterfraction
#print riverfraction
#waterfraction_max = waterfraction.max()

#waterfraction_idxMax = waterfraction.idxmax()

#riverfraction_max = riverfraction.max()

#print waterfraction_max
#print riverfraction_max

#river_contribution = riverfraction_max/waterfraction_max

#print river_contribution

#writer = ExcelWriter(r'E:\river_fraction_percentage.xlsx')
#river_contribution.to_excel(writer,'Sheet1')
#writer.save()

##----------------END-------------------#



##--------------calculate mean lake and river depth --------------##
lakedepth = df_lakevolume/lakearea
riverdepth = df_rivervolume/riverarea


print lakedepth
print riverdepth

mean_lakedepth = lakedepth.mean(axis = 1)
print mean_lakedepth
print np.mean(mean_lakedepth)  #1.16 m
mean_riverdepth = riverdepth.mean(axis = 1)
print np.ma.masked_invalid(mean_riverdepth).mean()   # 0.54 m

print 'done'


##--------------calculate lake volume contribution to water volume  --------------##

df_watervolume_max = df_watervolume.max()
df_lakevolume_max = df_lakevolume.max()

print df_watervolume_max

print df_lakevolume_max

lake_contribution = df_lakevolume_max/df_watervolume_max

print lake_contribution

writer = ExcelWriter(r'E:\lake_volume_percentage.xlsx')
lake_contribution.to_excel(writer,'Sheet1')
writer.save()



##---------------------END------------------------##










