# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot lake/river information and MAR smb"
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from pandas.io.excel import ExcelWriter

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2

def create_waterarea_fraction_DF(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    resolution = 30
    waterfraction = waterarea*resolution*resolution/(1000000*area.values)
    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)
    return waterfraction


def create_river_lake_DF(river_lake_stat_file):

    dbf = ps.open(river_lake_stat_file)
    d = dbf.by_col_array(dbf.header)  # my solution is better than the last sentence
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    types = df.index[:]
    areaIdx = 'area'
    lakeareaIdx = [ t for t in types if 'Larea' in t]
    lakecountIdx = [ t for t in types if 'Lcount' in t]
    riverIdx = [ t for t in types if 'Rlength' in t]

    #Use .iloc for integer based indexing and .loc for label based indexing.
    area = df.loc[areaIdx]
    lakearea = df.loc[lakeareaIdx]
    lakecount = df.loc[lakecountIdx]
    riverlength = df.loc[riverIdx]

    lakearea_dayIdx = lakearea.index[:]
    lakearea_dayIdx = [int(day[:3]) for day in lakearea_dayIdx]
    river_dayIdx = riverlength.index[:]
    river_dayIdx = [int(day[:3]) for day in river_dayIdx]

    lakefraction = lakearea/area.values
    drainage_density = riverlength/area.values

    lakecount.index = lakearea_dayIdx
    lakefraction.index = lakearea_dayIdx
    drainage_density.index = river_dayIdx

    lakecount = lakecount.sort_index(ascending=True)
    lakefraction = lakefraction.sort_index(ascending=True)
    drainage_density = drainage_density.sort_index(ascending=True)


    lakearea.index = lakearea_dayIdx
    lakearea = lakearea.sort_index(ascending=True)
    riverlength.index = river_dayIdx
    riverlength = riverlength.sort_index(ascending=True)

    return lakecount,lakearea,lakefraction,riverlength,drainage_density


def cal_lake_drainage_ratio(lakecount):
    lakeCount_DOY214 = lakecount.loc[214]
    print lakeCount_DOY214
    lakeCount_maximum = lakecount.max().to_frame().T
    print lakeCount_maximum

    #lakeCount_DOY214.loc['total'] = lakeCount_DOY214.sum()
    #lakeCount_maximum.loc['total'] = lakeCount_maximum.sum()


    drainage_ration = ((lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
    return drainage_ration


def plot_lake_drainage_averageArea(lakecount,drainage_ratio,ave_lake_area,large_lakecount,large_drainage_ratio,ave_large_lake_area):

    #set plotting environments
    matplotlib.style.use('classic')
    matplotlib.rcParams.update({'font.size': 25})
    matplotlib.rcParams['axes.linewidth'] = 2
    matplotlib.rcParams['xtick.major.size'] = 20
    matplotlib.rcParams['xtick.major.width'] = 2
    matplotlib.rcParams['ytick.major.size'] = 20
    matplotlib.rcParams['ytick.major.width'] = 2
    matplotlib.rcParams['lines.linewidth'] = 3
    matplotlib.rcParams['lines.markersize'] = 12

    f, (ax1,ax2,ax3) = plt.subplots(3)

    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)

    lakecount.plot(ax=ax1,legend=False,marker = 'o',cmap=cmap)
    ax1.set_xlabel('DOY')
    ax1.set_ylabel('All lake count')
    ax1.annotate('(a)',size = 20,xy=get_axis_limits(ax1))
    ax1.set_yticks(np.arange(0,500,100))
    ax1.legend(loc='upper left',fontsize=20)
    
    large_lakecount.plot(ax=ax2,legend=False,marker = 'o',cmap=cmap)
    ax2.set_xlabel('DOY')
    ax2.set_ylabel('Large lake count')
    ax2.annotate('(a)',size = 20,xy=get_axis_limits(ax2))
    ax2.set_yticks(np.arange(0,100,20))
    #ax2.legend(loc='upper right',fontsize=25)

    x=[800,1000,1200,1400,1600,1800]

    ax3.plot(x,drainage_ratio.loc[0].values,marker = 'd', color = 'r',zorder=10)
    ax3.plot(x,large_drainage_ratio.loc[0].values,marker = 'd', linestyle='--',markerfacecolor="None",markeredgecolor='r', markeredgewidth=2,color = 'r',zorder=10)
    #ax3.set_ylim([0, 1.2])
    ax3.set_xlim([600, 2000])
    #ax3.set_yticks(np.arange(0,1.2,0.2))
    ax3.set_xticks(np.arange(600,2000,200))
    ax3.set_ylabel('Lake disappearance ratio', color = 'r')
    ax3.set_xlabel('Elevation (m)')
    ax3.yaxis.label.set_color('r')
    ax3.tick_params(axis='y', colors='r')


    ax31 = ax3.twinx() 
    (_, caps, _)= ax31.errorbar(x,ave_lake_area.loc['ave_area'].values,yerr= ave_lake_area.loc['std_area'].values, capsize=10, elinewidth=2, marker = 's', color = 'b')
    for cap in caps:
        cap.set_markeredgewidth(3)
    (plotlines, caps, _) = ax31.errorbar(x,ave_large_lake_area.loc['ave_area'].values,yerr= ave_large_lake_area.loc['std_area'].values, capsize=10, elinewidth=2, marker = 's', linestyle='--', markerfacecolor="None",markeredgecolor='b', markeredgewidth=2,color = 'b')
    for cap in caps:
        cap.set_markeredgewidth(3)
    #plotlines.set_linestyle('--')
    ax31.set_ylabel('Average lake area $\mathregular{(km^2)}$', color = 'b')
    ax31.yaxis.label.set_color('b')
    ax31.tick_params(axis='y', colors='b')

    #ax31.axhline(y=0.25,color='k', linestyle='--',linewidth=2,zorder=0)

    #process subplot title overlap
    hspace = .3
    plt.subplots_adjust( hspace=hspace )


    plt.show()

        
    f.savefig(r'E:\lake_drainage_2015.tif', format='tif', dpi=450)


#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 20})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 16
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 16
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 10


#Before run this script, rerun step 10 to get large lakes (>0.25 km2) and rerun step 11 to get large lake statistics

#output of step 11
large_river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_large_lake_statistics.dbf'

large_lakecount,large_lakearea,lakefraction,riverlength,drainage_density = create_river_lake_DF(large_river_lake_stat_file)


#calculate lake drainage ratio  = (lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
large_drainage_ratio = cal_lake_drainage_ratio(large_lakecount)

#calculate average lake area
ave_large_lake_area = (large_lakearea/large_lakecount)
ave_large_lake_area.loc['ave_area']=ave_large_lake_area.mean()
ave_large_lake_area.loc['std_area']=ave_large_lake_area.std()

print ave_large_lake_area.loc['ave_area']
print ave_large_lake_area.loc['std_area']

print np.mean(ave_large_lake_area.loc['ave_area'])
print np.mean(ave_large_lake_area.loc['std_area'])

#output of step 11
river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_lake_statistics.dbf'

lakecount,lakearea,lakefraction,riverlength,drainage_density = create_river_lake_DF(river_lake_stat_file)


#calculate lake drainage ratio  = (lakeCount_maximum - lakeCount_DOY214) / lakeCount_maximum)
drainage_ratio = cal_lake_drainage_ratio(lakecount)

#calculate average lake area
ave_lake_area = (lakearea/lakecount)
ave_lake_area.loc['ave_area']=ave_lake_area.mean()
ave_lake_area.loc['std_area']=ave_lake_area.std()

print ave_lake_area.loc['ave_area']
print ave_lake_area.loc['std_area']



print 'done...'

plot_lake_drainage_averageArea(lakecount,drainage_ratio,ave_lake_area,large_lakecount,large_drainage_ratio,ave_large_lake_area)












